package com.bipt.intelligentapplicationorchestrationservice.controller;

import com.bipt.intelligentapplicationorchestrationservice.pojo.AlgorithmInfo;
import com.bipt.intelligentapplicationorchestrationservice.pojo.OptResult;
import com.bipt.intelligentapplicationorchestrationservice.service.AlgorithmInfoService;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
@Tag(name ="算法创建相关接口")
@RestController
@RequestMapping("/api/algorithm")
@Slf4j
@CrossOrigin(origins = "http://localhost:3000")
public class AlgorithmInfoController {

    @Autowired
    private AlgorithmInfoService algorithmInfoService;

    @Autowired
    private ObjectMapper objectMapper;

    @GetMapping("/{id}")
    public ResponseEntity<AlgorithmInfo> getById(@PathVariable Long id) {
        AlgorithmInfo algorithmInfo = algorithmInfoService.getById(id);
        return algorithmInfo != null ? 
            ResponseEntity.ok(algorithmInfo) : 
            ResponseEntity.notFound().build();
    }

    @GetMapping("/name/{algorithmName}")
    public ResponseEntity<AlgorithmInfo> getByName(@PathVariable String algorithmName) {
        AlgorithmInfo algorithmInfo = algorithmInfoService.getByName(algorithmName);
        return algorithmInfo != null ? 
            ResponseEntity.ok(algorithmInfo) : 
            ResponseEntity.notFound().build();
    }

    @GetMapping("/type/{algorithmType}")
    public ResponseEntity<List<AlgorithmInfo>> getByType(@PathVariable String algorithmType) {
        List<AlgorithmInfo> algorithmInfos = algorithmInfoService.getByType(algorithmType);
        return ResponseEntity.ok(algorithmInfos);
    }

    @PutMapping("/{id}")
    public ResponseEntity<String> update(@PathVariable Long id, @RequestBody AlgorithmInfo algorithmInfo) {
        algorithmInfo.setId(id);
        if (!algorithmInfoService.validateAlgorithmInfo(algorithmInfo)) {
            return ResponseEntity.badRequest().body("Invalid algorithm information");
        }
        
        boolean success = algorithmInfoService.update(algorithmInfo);
        return success ? 
            ResponseEntity.ok("Update successful") : 
            ResponseEntity.badRequest().body("Update failed");
    }

    @DeleteMapping("/{id}")
    public ResponseEntity<String> delete(@PathVariable Long id) {
        boolean success = algorithmInfoService.delete(id);
        return success ? 
            ResponseEntity.ok("Delete successful") : 
            ResponseEntity.badRequest().body("Delete failed");
    }

    @PostMapping(consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @Operation(summary ="算法创建")
    public OptResult save(@RequestParam("algorithm") String algorithmJson,
                          @RequestPart(value = "algorithmFile") MultipartFile file) {
        try {
            AlgorithmInfo algorithmInfo = objectMapper.readValue(algorithmJson, AlgorithmInfo.class);
            log.info("新增算法: {}, 文件: {}", algorithmInfo, (file != null ? file.getOriginalFilename() : "无文件"));
            algorithmInfoService.save(algorithmInfo, file);
            return OptResult.success("算法创建成功");
        } catch (IOException e) {
            log.error("JSON转换失败", e);
            return OptResult.error("新增算法失败: " + e.getMessage());
        } catch (Exception e) {
            log.error("新增算法失败", e);
            return OptResult.error("新增算法失败: " + e.getMessage());
        }
    }

    /**
     * 根据算法名称模糊查询算法信息
     * @param keyword 模糊查询关键词
     * @return 符合条件的算法信息列表
     */
    @GetMapping("/search")
    public ResponseEntity<List<AlgorithmInfo>> searchByName(@RequestParam String keyword) {
        List<AlgorithmInfo> algorithmInfos = algorithmInfoService.getByNameLike(keyword);
        return ResponseEntity.ok(algorithmInfos);
    }
    /**
     * 算法运行
     */
    @PostMapping("/run/{id}")
    @Operation(summary = "运行")
    public OptResult run(@PathVariable Long id,@RequestBody String param){
        log.info("运行",id);
        String result = algorithmInfoService.run(id,param);
        return OptResult.success("运行成功"+result);
    }
    /**
     * 前端列表返回算法名称
     */
    @GetMapping("/names")
    @Operation(summary = "列表返回算法名称")
    public List<String> getNames(){
        return algorithmInfoService.getAllNames();
    }

} 