package com.bipt.intelligentapplicationorchestrationservice.controller;

import com.bipt.intelligentapplicationorchestrationservice.pojo.*;
import com.bipt.intelligentapplicationorchestrationservice.service.DatasetService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;
@Tag(name ="数据集相关接口")
@RestController
@RequestMapping("/dataset")
@Slf4j
@CrossOrigin(origins = "http://localhost:3000")
public class DatasetController {
    @Autowired
    private DatasetService datasetService;

    /**
     * 新增数据集（整合文件上传）
     * @param datasetDTO
     * @param file
     * @return
     */
    @Operation(summary = "新增数据集")
    @PostMapping(consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public OptResult save(
            @ModelAttribute("dataset") DatasetDTO datasetDTO,
            @RequestPart(value = "file", required = false) MultipartFile file) {
        log.info("新增数据集: {}, 文件: {}", datasetDTO, (file != null ? file.getOriginalFilename() : "无文件"));
        try {
            datasetService.save(datasetDTO, file);
            return OptResult.success();
        } catch (Exception e) {
            log.error("新增数据集失败", e);
            return OptResult.error("新增数据集失败: " + e.getMessage());
        }
    }

    /**
     * 分页查询
     * @param dataSetPageQueryDTO
     * @return
     */
    @Operation(summary ="分页查询")
    @GetMapping("/page")
    public OptResult<PageResult> page(DatasetPageQueryDTO dataSetPageQueryDTO) {
        log.info("数据集分页查询:{}", dataSetPageQueryDTO);
        PageResult pageResult = datasetService.pageQuery(dataSetPageQueryDTO);
        return OptResult.success(pageResult);
    }

    /**
     * 修改数据集
     * @param datasetDTO
     * @return
     */
    @Operation(summary ="修改数据集")
    @PutMapping(consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public OptResult update(@ModelAttribute("dataset") DatasetDTO datasetDTO,
                            @RequestPart(value = "file", required = false) MultipartFile file){
        log.info("修改数据集{}, 文件: {}", datasetDTO, (file != null ? file.getOriginalFilename() : "无文件"));
        datasetService.update(datasetDTO,file);
        return OptResult.success();
    }



    /**
     * 批量删除数据集
     * @param datasetIds 数据集ID列表
     * @return 操作结果
     */
    @Operation(summary ="删除数据集")
    @DeleteMapping
    public OptResult<String> deleteBatch(@RequestBody List<Long> datasetIds) {
        log.info("批量删除数据集，ID列表：{}", datasetIds);
        datasetService.deleteBatch(datasetIds);
        return OptResult.success("批量删除成功");
    }

    /**
     * 下载数据集
     * @param datasetId 数据集ID
     * @return 数据集文件
     */
    @Operation(summary = "下载数据集")
    @GetMapping("/download/{datasetId}")
    public ResponseEntity<byte[]> downloadDataset(@PathVariable Long datasetId) {
        log.info("下载数据集，ID：{}", datasetId);
        try {
            // TODO: 调用分布式存储系统的接口获取数据集文件的输入流
            InputStream inputStream = datasetService.downloadDataset(datasetId);
            if (inputStream == null) {
                return new ResponseEntity<>(HttpStatus.NOT_FOUND);
            }
            byte[] fileBytes = inputStream.readAllBytes();
            inputStream.close();

            // 设置响应头
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
            headers.setContentDispositionFormData("attachment", "dataset_" + datasetId + ".zip");

            return new ResponseEntity<>(fileBytes, headers, HttpStatus.OK);
        } catch (IOException e) {
            log.error("下载数据集失败", e);
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

}
