package com.bipt.intelligentapplicationorchestrationservice.service.Impl;

import com.bipt.intelligentapplicationorchestrationservice.mapper.AlgorithmInfoMapper;
import com.bipt.intelligentapplicationorchestrationservice.pojo.AlgorithmInfo;
import com.bipt.intelligentapplicationorchestrationservice.service.AlgorithmInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.time.LocalDateTime;
import java.util.List;

@Service
@Slf4j
public class AlgorithmInfoServiceImpl implements AlgorithmInfoService {

    @Autowired
    private AlgorithmInfoMapper algorithmInfoMapper;

    @Value("${algorithm.upload.dir:/tmp/algorithm-files/}") // 默认上传目录
    private String uploadDir;
    @Override
    public AlgorithmInfo getById(Long id) {
        return algorithmInfoMapper.selectById(id);
    }

    @Override
    public AlgorithmInfo getByName(String algorithmName) {
        return algorithmInfoMapper.selectByName(algorithmName);
    }

    @Override
    public List<AlgorithmInfo> getByType(String algorithmType) {
        return algorithmInfoMapper.selectByType(algorithmType);
    }

    @Override
    @Transactional
    public boolean update(AlgorithmInfo algorithmInfo) {
        if (!validateAlgorithmInfo(algorithmInfo)) {
            return false;
        }
        return algorithmInfoMapper.update(algorithmInfo) > 0;
    }

    @Override
    @Transactional
    public boolean delete(Long id) {
        return algorithmInfoMapper.deleteById(id) > 0;
    }

    @Override
    public boolean validateAlgorithmInfo(AlgorithmInfo algorithmInfo) {
        if (algorithmInfo == null) {
            return false;
        }
        
        if (!StringUtils.hasText(algorithmInfo.getAlgorithmName()) ||
            !StringUtils.hasText(algorithmInfo.getAlgorithmFile()) ||
            !StringUtils.hasText(algorithmInfo.getAlgorithmType())) {
            return false;
        }
        
        return true;
    }

    @Override
    @Transactional
    public void save(AlgorithmInfo algorithmInfo, MultipartFile file) {
        String algorithmName = algorithmInfo.getAlgorithmName();
        // 检查同名算法
        AlgorithmInfo duplicateName = algorithmInfoMapper.selectByName(algorithmName);
        if (duplicateName != null) {
            throw new RuntimeException("算法已存在，请去修改算法");
        }

        // 只接收文件但不进行保存操作
        if (file != null && !file.isEmpty()) {
            log.info("已接收文件: {}", file.getOriginalFilename());
            log.info("文件大小: {} 字节", file.getSize());
            log.info("文件类型: {}", file.getContentType());
            // 临时设置一个空路径（避免数据库保存空值）
            //todo 保存到分布式存储
            algorithmInfo.setAlgorithmFile("");
        }

        algorithmInfo.setCreateTime(LocalDateTime.now());
        // 保存算法信息到数据库（注意：此时algorithmFile字段为空）
        algorithmInfoMapper.insert(algorithmInfo);
    }

    @Override
    public String run(Long id, String param) {
        String file = algorithmInfoMapper.getFileById(id);
        StringBuilder result = new StringBuilder(); // 用于存储结果

        try {
            // 构建命令，将 param 作为参数传递给 Python 脚本
            ProcessBuilder pb = new ProcessBuilder("python", file, param);
            Process process = pb.start();

            // 读取标准输出（脚本执行结果）
            BufferedReader reader = new BufferedReader(
                    new InputStreamReader(process.getInputStream()));
            String line;
            while ((line = reader.readLine()) != null) {
                result.append(line).append("\n");
            }

            // 读取错误输出
            BufferedReader errorReader = new BufferedReader(
                    new InputStreamReader(process.getErrorStream()));
            String errorLine;
            while ((errorLine = errorReader.readLine()) != null) {
                result.append("Error: ").append(errorLine).append("\n");
            }

            int exitCode = process.waitFor();
            result.append("Exit Code: ").append(exitCode);

        } catch (Exception e) {
            result.append("执行异常: ").append(e.getMessage());
            e.printStackTrace();
        }

        return result.toString(); // 返回完整结果
    }

    @Override
    public List<String> getAllNames() {
        return algorithmInfoMapper.getAllNames();
    }

    /**
     * 模糊查询
     * @param keyword
     * @return
     */
    @Override
    public List<AlgorithmInfo> getByNameLike(String keyword) {
        return algorithmInfoMapper.selectByNameLike(keyword);
    }
} 